/* matrizTransicion.c - Matriz de transicion */
#include <stdio.h>
#include <stdlib.h>
#include <limits.h>
#include <string.h>
#include <time.h>

#define NUMNOTAS 4

void MIDI2Note(int MIDI, char Note[]);
int transicion(int j, float matriz[][NUMNOTAS]);
void textoAmidi(char *archivo_texto, char *archivo_midi);
void uso(void);

int main(int argc, char *argv[]){

	int i, j, k, cantidadDeNotas, primeraNota;
	float matriz[NUMNOTAS][NUMNOTAS] = {{0.0, 0.3, 0.0, 0.0}, {1.0, 0.3, 0.3, 0.0}, {0.0, 0.4, 0.6, 0.5}, {0.0, 0.0, 0.1, 0.5}};
	int midi[NUMNOTAS] = {60, 62, 64, 67};
	int velocity = 127;
	float tiempo = 1, duracion = 0.25;
	FILE *fPtr; /* puntero a archivo */
	char Nota[4], *temp="temp.txt"; /* nombre de archivo temporal */;
	
	if (argc != 4){
		uso();
	}
	else{

		cantidadDeNotas = atoi(argv[2]);
		primeraNota = atoi(argv[3]);
	
		srand(time(NULL));
		
		printf("Notas MIDI: \n");
		for (i = 0; i < NUMNOTAS; i++){
			printf("%2d\t", midi[i]);
		}
		printf("\n");
		for (i = 0; i < NUMNOTAS; i++){
			MIDI2Note(midi[i],Nota);
			printf("%2s\t", Nota);
		}
		printf("\n");
		printf("\n");
		printf("Matriz de transicion: \n");
		printf("\t");
		for (i = 0; i < NUMNOTAS; i++){
			MIDI2Note(midi[i],Nota);
			printf("%3s \t", Nota);
		}
		printf("\n");
	
		for (i = 0; i < NUMNOTAS; i++){
			MIDI2Note(midi[i],Nota);
			printf("%2s \t", Nota);
			for (j = 0; j < NUMNOTAS; j++){
				printf("%1.1f\t", matriz[i][j]);
			}
			printf("\n");
		}
		printf("\n");
		
		if ((fPtr = fopen(temp, "w")) == NULL)
			printf("No se pudo crear archivo\n");
		else{
			j=primeraNota; /* Primera nota */
			if (j < NUMNOTAS){
			  for (k = 0; k < cantidadDeNotas; k++){
				printf("%2d ",midi[j]);
				fprintf(fPtr, "note\t%.3f\t%.3f\t%d\t%d\n", tiempo, duracion, midi[j], velocity);
				tiempo+=duracion;
				j = transicion(j, matriz);
			  } 
			printf("\n");
			}
			fclose(fPtr);
			textoAmidi(temp, argv[1]);
		}
	}
	
	return 0;
}

// escribe la nota a partir del numero MIDI	
void MIDI2Note(int MIDI, char Note[]){

  int octava, offset;
  char *notas[12] = {"C","C#","D","D#","E","F","F#","G","G#","A","A#","B"};
  char soctava[2];

  octava = MIDI / 12 - 1;
  offset = MIDI % 12;

  sprintf(soctava, "%d", octava);
  strcpy(Note, notas[offset]);
  strcat(Note, soctava);

}

int transicion(int j, float matriz[][NUMNOTAS]){
	float p, umbral = 0;
	int i, num;
	
	p = (float)rand() / RAND_MAX;
	
	for (i = 0; i < NUMNOTAS; i++){
		umbral += matriz[i][j];
		if (p <= umbral)
			return i;
	}
	
}


void textoAmidi(char *archivo_texto, char *archivo_midi){
     
     char comando[PATH_MAX] = "./text2midi ";
     
     /* Se construye el comando a ejecutar */
     strcat(comando, archivo_texto);
     strcat(comando, " ");
     strcat(comando, archivo_midi);

     /* Se hace una llamada al sistema */
     system(comando);
}

void uso(void){
	printf("\nPrograma para generar secuencias de notas aleatorias a partir de una matriz de transicion.\nEl programa despliega la matriz de transicion, la secuencia de notas generada y produce un archivo midi. \nLa cantidad de notas de la secuencia y la nota inicial se indican como parametros de linea de comando.\n\n");
	printf("uso: matrizTransicion archivo_midi numero_notas nota_inicial\n\n");
	printf("archivo_midi - nombre del archivo midi de salida\n");
	printf("numero_notas - numero de notas a generar\n");
	printf("nota_inical  - indice de la nota inicial de la secuencia (comenzando en 0)\n\n");
	
	printf("Ejemplo: \n");
	printf("./matrizTransicion secuencia.mid 24 0\n");
	printf("Se generan 24 notas comenzando en la nota de indice 0.\n\n");

	
}
