/* matrizTransicion.c - Matriz de transicion */
#include <stdio.h>
#include <stdlib.h>
#include <limits.h>
#include <string.h>
#include <time.h>

#define NUMNOTAS 12
#define NUMFIGURAS 4
#define COMPASES 7

#define MATRIZ {{  0.0,   0.0, 0.0625,    0.0,    0.0,    0.0,    0.0,    0.0,    0.0,    0.0,    0.0,    0.0},                           {  0.0,   0.0, 0.0625, 0.0625,    0.0,    0.0,    0.0,    0.0,    0.0,    0.0,    0.0,    0.0},                           {  1.0,   1.0, 0.1250, 0.3750, 0.1250,    0.0,    0.0, 0.0625, 0.0625,    0.0,    0.0,    0.0},                           {  0.0,   0.0, 0.3125, 0.1875, 0.2500,    0.0,    0.0,    0.0,    0.0,    0.0,    0.0,    0.0},                           {  0.0,   0.0, 0.1875, 0.2500, 0.3125, 0.2500,    0.0, 0.3125, 0.0625,    0.0,    0.0,    0.0},                           {  0.0,   0.0, 0.0625,    0.0, 0.1250, 0.1875,    0.0, 0.0625, 0.0625,    0.0,    0.0,    0.0},                           {  0.0,   0.0,    0.0,    0.0,    0.0,    0.0,    0.0, 0.0625,    0.0,    0.0,    0.0,    0.0},                           {  0.0,   0.0, 0.0625, 0.0625, 0.1250, 0.3750,    1.0, 0.2500, 0.5625,    0.0, 0.2500, 0.3750},                           {  0.0,   0.0,    0.0,    0.0, 0.0625, 0.1875,    0.0, 0.1875, 0.1250, 0.5000, 0.4375,    0.0},                           {  0.0,   0.0, 0.0625,    0.0,    0.0,    0.0,    0.0,    0.0,    0.0,    0.0, 0.1875, 0.6250},                           {  0.0,   0.0, 0.0625, 0.0625,    0.0,    0.0,    0.0, 0.0625, 0.1250, 0.5000, 0.0625,    0.0},                           {  0.0,   0.0,    0.0,    0.0,    0.0,    0.0,    0.0,    0.0,    0.0,    0.0, 0.0625,    0.0}} 

#define DURFIGURAS {{ 1.0,   0.0,   0.0,   0.0},                                                                                              { 0.5,   0.5,   0.0,   0.0},                                                                                              {0.25,  0.25,  0.25,  0.25},                                                                                              { 0.5,  0.25,  0.25,   0.0},                                                                                              {0.25,  0.25,   0.5,   0.0},                                                                                              {0.25,   0.5,  0.25,   0.0},                                                                                              { 1.0,   0.0,   0.0,   0.0}}

void MIDI2Note(int MIDI, char Note[]);
int transicion(int j, float matriz[][NUMNOTAS]);
void textoAmidi(char *archivo_texto, char *archivo_midi);
int compas(float probcompases[]);
void uso(void);

int main(int argc, char *argv[]){

	int i, j, k, cantidadDeCompases, primeraNota, indCompas, indMidi;
	float matriz[NUMNOTAS][NUMNOTAS] = MATRIZ;
	float probcompases[COMPASES] = {0.125, 0.250, 0.125, 0.125, 0.125, 0.125, 0.125};
	float duraciones[COMPASES][4] = DURFIGURAS;
	
	float suma;
	int midi[NUMNOTAS] = {59, 61, 62, 64, 66, 67, 68, 69, 71, 73, 74, 76};
	int velocity = 127;
	float tiempo = 1, duracion = 0.25;
	FILE *fPtr; /* puntero a archivo */
	char Nota[4], *temp="temp.txt"; /* nombre de archivo temporal */;
	
	if (argc != 4){
		uso();
	}
	else{

		cantidadDeCompases = atoi(argv[2]);
		primeraNota = atoi(argv[3]);
	
		srand(time(NULL));
		
		/* Se muestran las notas */
		printf("\nNotas MIDI: \n");
		for (i = 0; i < NUMNOTAS; i++){
			printf("%3d\t", midi[i]);
		}
		printf("\n");
		for (i = 0; i < NUMNOTAS; i++){
			MIDI2Note(midi[i],Nota);
			printf("%3s\t", Nota);
		}
		printf("\n(D mayor con G#)\n\n");
		/* Se muestra la matriz de transicion */
		printf("Matriz de transicion: \n");
		printf("\t");
		for (i = 0; i < NUMNOTAS; i++){
			MIDI2Note(midi[i],Nota);
			printf("%6s \t", Nota);
		}
		printf("\n");
		for (i = 0; i < NUMNOTAS; i++){
			MIDI2Note(midi[i],Nota);
			printf("%6s\t", Nota);
			for (j = 0; j < NUMNOTAS; j++){
				printf("%1.4f\t", matriz[i][j]);
			}
			printf("\n");
		}
		/* Se verifica que la probabilidad sume 1 en cada columna */
		printf("Total:\t");
		for (j = 0; j < NUMNOTAS; j++){
			suma = 0;
			for (i = 0; i < NUMNOTAS; i++){
				suma += matriz[i][j];
			}
			printf("%1.4f\t",suma);
		}
		printf("\n\n");
		
		/* Se muestran los posibles patrones ritmicos para un compas de 4/4 y su probabilidad */
		printf("Patrones ritmicos para un compas de 4/4 y su probabilidad: \n");
		printf("\tprob.\tduracion figuras respecto compas\n");
		suma = 0;
		for (i = 0; i < COMPASES; i++){
			j = 0;
			printf("%5d\t%1.3f\t", i, probcompases[i]);
			suma += probcompases[i];
			while (duraciones[i][j] > 0 && j < NUMFIGURAS){
				printf("%1.2f\t", duraciones[i][j]);
				j++;
			}
			printf("\n");
		}
		printf("Total\t%1.3f\n", suma);
		printf("(nota: el ultimo patron corresponde a un silencio de redonda)\n\n");
		
		if ((fPtr = fopen(temp, "w")) == NULL)
			printf("No se pudo crear archivo\n");
		else{
			/* Se generan los compases */
			indMidi = primeraNota;
			for (i = 0; i < cantidadDeCompases; i++){ 
				/* se sortea un compas */
				indCompas = compas(probcompases);
				/* se generan la cantidad de notas de ese compas */
				/* si el compas es el 6 (silencio de redonda) solo se incrementa tiempo */
				if (indCompas == 6){
				   tiempo += duraciones[indCompas][0];
				   printf("%1.2f / SIL", duraciones[indCompas][0]);
				}
				else{
				   j = 0;
				   while (duraciones[indCompas][j] > 0 && j < NUMFIGURAS){
				        duracion = duraciones[indCompas][j];
					MIDI2Note(midi[indMidi],Nota);
				   	printf("%1.2f / %s\t", duraciones[indCompas][j], Nota);
					fprintf(fPtr, "note\t%.3f\t%.3f\t%d\t%d\n", tiempo, duracion, midi[indMidi], velocity);
				   	j++;
					tiempo += duracion;
					indMidi = transicion(indMidi, matriz);
				   }	
				}
				printf("\n");
			}
			printf("\n");	
			fclose(fPtr);
			textoAmidi(temp, argv[1]);
		}
	}
	
	return 0;
}

// escribe la nota a partir del numero MIDI	
void MIDI2Note(int MIDI, char Note[]){

  int octava, offset;
  char *notas[12] = {"C","C#","D","D#","E","F","F#","G","G#","A","A#","B"};
  char soctava[2];

  octava = MIDI / 12 - 1;
  offset = MIDI % 12;

  sprintf(soctava, "%d", octava);
  strcpy(Note, notas[offset]);
  strcat(Note, soctava);

}

int transicion(int j, float matriz[][NUMNOTAS]){
	float p, umbral = 0;
	int i;
	
	p = (float)rand() / RAND_MAX;
	
	for (i = 0; i < NUMNOTAS; i++){
		umbral += matriz[i][j];
		if (p <= umbral)
			return i;
	}
}

int compas(float probcompases[]){
	float p, umbral = 0;
	int i;
	
	p = (float)rand() / RAND_MAX;
	
	for (i = 0; i < COMPASES; i++){
		umbral += probcompases[i];
		if (p <= umbral)
			return i;
	}
}


void textoAmidi(char *archivo_texto, char *archivo_midi){
     
     char comando[PATH_MAX] = "./text2midi ";
     
     /* Se construye el comando a ejecutar */
     strcat(comando, archivo_texto);
     strcat(comando, " ");
     strcat(comando, archivo_midi);

     /* Se hace una llamada al sistema */
     system(comando);
}

void uso(void){
	printf("\nPrograma para generar secuencias de notas aleatorias a partir de una matriz de transicion.\nEl programa despliega la matriz de transicion, la secuencia de notas generada y produce un archivo midi. \nLa cantidad de notas de la secuencia y la nota inicial se indican como parametros de linea de comando.\n\n");
	printf("uso: matrizTransicion archivo_midi numero_notas nota_inicial\n\n");
	printf("archivo_midi    - nombre del archivo midi de salida\n");
	printf("numero_compases - numero de compases a generar\n");
	printf("nota_inical     - indice de la nota inicial de la secuencia (comenzando en 0)\n\n");
	
	printf("Ejemplo: \n");
	printf("./matrizTransicion secuencia.mid 12 0\n");
	printf("Se generan 12 compases comenzando en la nota de indice 0.\n\n");

	
}
